#ifndef GAME_STATE_H
#define GAME_STATE_H

#include "stdint.h"
#include "stdbool.h"

#ifdef __cplusplus
extern "C" {
#endif

enum Board_States_t {
    LIGHT_OFF = 0u,
    POTENTIAL_MOVE,
    POTENTIAL_TAKE,
    SUGGESTED_MOVE,
    ERROR_MOVE,
    PIECE_ORIGIN,
    PIECE_NEEDS_TO_BE_HERE,
    POTENTIAL_CASTLE,
    PIECE_NEEDS_TO_BE_REMOVED,
    CONVERTING_PAWN,
    EN_PASSANT,
    EN_PASSANT_REMOVE,
};

#define PAWN_WHITE      0u
#define PAWN_BLACK      1u
#define KING_WHITE      2u
#define KING_BLACK      3u
#define ROOK_WHITE      4u
#define ROOK_BLACK      5u
#define KNIGHT_WHITE    6u
#define KNIGHT_BLACK    7u
#define BISHOP_WHITE    8u
#define BISHOP_BLACK    9u
#define QUEEN_WHITE     10u
#define QUEEN_BLACK     11u
#define SQUARE_EMPTY    12u

#define WHITE_TURN true
#define BLACK_TURN false

typedef enum {
    BEGINNING = 0,
    IN_PROGRESS,
    FINALIZING,
}Turn_State_t;

typedef struct {
    bool game_over;
    bool error_detected;
    bool player_turn;
    Turn_State_t turn_state;
    uint8_t board_state[8*8];
    uint8_t board_pieces[8*8];
    uint8_t selected_piece;
    uint8_t selected_piece_origin;
    bool castling_allowed[2u][2u];
    int en_passant;
}Game_State_t;

void game_state_init(void);
void Board_Changed(uint8_t current_binary_board[8]);
bool Set_Light(uint8_t piece, uint8_t row, uint8_t column, uint8_t state);
void clear_board_state(void);
Game_State_t * Board_get_game_state(void);

#ifdef __cplusplus
}
#endif
#endif
