#include "pawn.h"
#include "game_state.h"
#include "piece_logic.h"

static bool Converting_Pawn = false;
static uint8_t Converting_Pawn_Row_Col[2];
static uint8_t Pawn_Converted_To = QUEEN_WHITE;

void Check_If_Converting_Pawn(uint8_t row, uint8_t column, Game_State_t * game_state)
{
    uint8_t white_black_idx = game_state->player_turn ? 0u : 1u;
    Converting_Pawn = false;

    if((game_state->selected_piece == PAWN_WHITE) || (game_state->selected_piece == PAWN_BLACK))
    {
        if((row == 0u) || (row == 7u))
        {
            game_state->selected_piece = game_state->player_turn ? QUEEN_WHITE : QUEEN_BLACK;
            Pawn_Converted_To = game_state->selected_piece;
            Converting_Pawn = true;
            Converting_Pawn_Row_Col[0] = row;
            Converting_Pawn_Row_Col[1] = column;
        }
    }
}

bool Converting_Pawn_If_Applicable(uint8_t row, uint8_t column, Game_State_t * game_state)
{
    bool ret_val = false;
    if(Converting_Pawn)
    {
        if((row == Converting_Pawn_Row_Col[0]) &&
            (Converting_Pawn_Row_Col[1] == column))
        {
            //Putting the piece down on the board
            if(game_state->board_pieces[row*8+column] == SQUARE_EMPTY)
            {
                game_state->board_pieces[row*8+column] = Pawn_Converted_To;
                game_state->board_state[row*8+column] = LIGHT_OFF;
            }
            //Picking the piece back up to toggle through the options
            else
            {
                game_state->board_pieces[row*8+column] = SQUARE_EMPTY;
                game_state->board_state[row*8+column] = CONVERTING_PAWN;
                Pawn_Converted_To = Pawn_Converted_To - 2;
                if (Pawn_Converted_To < ROOK_WHITE)
                {
                    Pawn_Converted_To += 8u;
                }

            }
            ret_val = true;
        }

    }

    return ret_val;
}



/**
 * @brief  Function for marking potential moves for pawns.
 * @param  row: row to move to
 * @param  column: column to move to
 * @retval None
 */
bool pawn_move(uint8_t piece, uint8_t row, uint8_t column, Game_State_t * game_state)
{
    bool ret_val = false;
    if (game_state->board_pieces[row*8+column] == SQUARE_EMPTY)
    {
        ret_val = Set_Light(piece, row, column, POTENTIAL_MOVE);
    }
    return ret_val;
}

/**
 * @brief  Marking a piece for taking.
 * @param  row: Row ofcoarse
 * @param  column: Column obviously
 * @retval None
 */
bool pawn_take(uint8_t piece, uint8_t row, uint8_t column, Game_State_t * game_state)
{
    bool ret_val = false;
    if ((game_state->board_pieces[row*8+column] < SQUARE_EMPTY) && opposite_teams(piece, game_state->board_pieces[row*8+column]))
    {
        ret_val = Set_Light(piece, row, column, POTENTIAL_TAKE);
    }
    else if (game_state->en_passant == row*8+column) {
        ret_val = Set_Light(piece, row, column, EN_PASSANT);
    }
    return ret_val;
}

void Mark_En_Passant_Target(uint8_t location, Game_State_t * game_state)
{
    if ((game_state->selected_piece_origin / 8 == 1) || (game_state->selected_piece_origin / 8 == 6))
    {
        if (((game_state->board_pieces[location] == PAWN_BLACK) && ((location / 8) == 3)) ||
                ((game_state->board_pieces[location] == PAWN_WHITE) && ((location / 8) == 4)))
        {
            int offset = game_state->player_turn ? 8 : -8;
            game_state->en_passant = location + offset;
            return;
        }
    }
    game_state->en_passant = -1;
}

